package com.led.edison;

import android.os.Handler;
import android.support.v7.app.ActionBarActivity;
import android.os.Bundle;
import android.view.Menu;
import android.view.MenuItem;

import android.bluetooth.BluetoothSocket;
import android.content.Intent;
import android.view.View;
import android.widget.Button;
import android.widget.TextView;
import android.widget.Toast;
import android.app.ProgressDialog;
import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothDevice;
import android.os.AsyncTask;

import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.util.UUID;


public class MainControl extends ActionBarActivity {

    Button btnOn, btnDis;
    TextView messagetext;
    TextView messagecounter;
    String address = null;
    private ProgressDialog progress;
    BluetoothAdapter myBluetooth = null;
    BluetoothSocket btSocket = null;
    private boolean isBtConnected = false;
    //SPP UUID. Look for it
    static final UUID myUUID = UUID.fromString("00001101-0000-1000-8000-00805F9B34FB");

    int CounterMess = 0;

    ReaderBT ReaderBTobj = new ReaderBT();     ;

    @Override
    protected void onCreate(Bundle savedInstanceState)
    {
        super.onCreate(savedInstanceState);

        Intent newint = getIntent();
        address = newint.getStringExtra(DeviceList.EXTRA_ADDRESS); //receive the address of the bluetooth device

        //view of the MainControl
        setContentView(R.layout.activity_led_control);

        //call the widgtes
        btnOn = (Button)findViewById(R.id.button2);
        btnDis = (Button)findViewById(R.id.button4);
        messagetext = (TextView)findViewById(R.id.textMessages);
        messagecounter = (TextView)findViewById(R.id.textCounterUpdate);

        new ConnectBT().execute(); //Call the class to connect

        //commands to be sent to bluetooth
        btnOn.setOnClickListener(new View.OnClickListener()
        {
            @Override
            public void onClick(View v)
            {
                SendPing();      //method to turn on
            }
        });


        btnDis.setOnClickListener(new View.OnClickListener()
        {
            @Override
            public void onClick(View v)
            {
                Disconnect(); //close connection
            }
        });

    }

    private boolean WriteBluetooth(String dataToWrite)
    {
        if (btSocket!=null)
        {
            try
            {
                btSocket.getOutputStream().write((byte)0x02);
                btSocket.getOutputStream().write(dataToWrite.getBytes());
                btSocket.getOutputStream().write((byte)0x03);

                return true;
            }
            catch (IOException e)
            {
                msg("Error");

                return false;
            }
        }

        return false;
    }

    private void Disconnect()
    {
        if (btSocket!=null) //If the btSocket is busy
        {
            try
            {
                btSocket.close(); //close connection
            }
            catch (IOException e)
            { msg("Error");}
        }
        finish(); //return to the first layout

    }

    private void SendPing()
    {
        WriteBluetooth("PING".toString());
    }

    // fast way to call Toast
    private void msg(String s)
    {
        Toast.makeText(getApplicationContext(),s,Toast.LENGTH_LONG).show();
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        // Inflate the menu; this adds items to the action bar if it is present.
        getMenuInflater().inflate(R.menu.menu_led_control, menu);
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        // Handle action bar item clicks here. The action bar will
        // automatically handle clicks on the Home/Up button, so long
        // as you specify a parent activity in AndroidManifest.xml.
        int id = item.getItemId();

        //noinspection SimplifiableIfStatement
        if (id == R.id.action_settings) {
            return true;
        }

        return super.onOptionsItemSelected(item);
    }

    synchronized public void BluetoothMessageRead(String Message)
    {
        try
        {
            if (Message.length() >= 4)
            {
                String dataView = "";
                String cmd = Message.substring(0, 4);

                if ("ACQ ".equals(cmd)) {
                    int i;
                    String data = Message.substring(4);
                    String[] pairs = data.split(";");

                    for (i = 0; i < pairs.length; i++) {
                        String[] separated = pairs[i].split(":");

                        if (separated.length == 2) {
                            dataView += separated[0] + " = " + separated[1] + "\n";
                        }
                    }

                    messagetext.setText(dataView);

                    // TODO: your send data to favourite IoT clud server
                }
                messagecounter.setText(String.valueOf(++CounterMess));
            }
        }
        catch (Exception e)
        {
            msg("Error");
        }
    }

    public void BluetoothConnected(boolean Success)
    {
        ReaderBTobj.execute();
    }

    private class ReaderBT extends AsyncTask<Void, Void, Void>  // UI thread
    {
        public  Handler mHandler = new Handler();
        boolean ContinueRead = true;

        byte[] buffermessages  = new byte[1000];
        boolean PendingMess = false;
        int Index = 0;

        @Override
        protected void onPreExecute()
        {
        }

        @Override
        protected Void doInBackground(Void ... devices) //while the progress dialog is shown, the connection is done in background
        {
            byte[] bufferbluetooth  = new byte[1000];
            int numbytes;

            while(ContinueRead)
            {
                try
                {
                    if (isBtConnected)
                    {

                        while (btSocket.getInputStream().available() > 0) {
                            numbytes = btSocket.getInputStream().read(bufferbluetooth);

                            ParseMessage(bufferbluetooth, numbytes);

                        }
                    }
                }
                catch (Exception e)
                {
                    msg("Error");
                }
            }

            return null;
        }

        private void ParseMessage(byte[] data, int NumBytes)
        {
            int i;
            byte ele;

            for ( i = 0 ; i < NumBytes ; i++)
            {
                ele = data[i];

                if (ele == 0x02)
                {
                    Index = 0;
                    PendingMess = true;
                }
                else
                {
                    if (ele == 0x03)
                    {
                        if (PendingMess)
                        {
                            buffermessages[Index] = 0;

                            mHandler.post(new Runnable() {
                                public void run() {
                                    try {
                                        String temp = new String(buffermessages, "UTF-8");
                                        BluetoothMessageRead(temp);
                                    } catch (UnsupportedEncodingException ex) {

                                    }
                                }

                                ;
                            });

                            PendingMess = false;
                        }

                        Index = 0;
                    }
                    else
                    {
                        if (PendingMess)
                        {
                            if (Index < 100)
                            {
                                buffermessages[Index++] = ele;
                            }
                        }
                    }
                }
            }
        }


        @Override
        protected void onPostExecute(Void result) //after the doInBackground, it checks if everything went fine
        {
            super.onPostExecute(result);

        }
    }

    private class ConnectBT extends AsyncTask<Void, Void, Void>  // UI thread
    {
        private boolean ConnectSuccess = true; //if it's here, it's almost connected

        @Override
        protected void onPreExecute()
        {
            progress = ProgressDialog.show(MainControl.this, "Connecting...", "Please wait!!!");  //show a progress dialog
        }

        @Override
        protected Void doInBackground(Void... devices) //while the progress dialog is shown, the connection is done in background
        {
            try
            {
                if (btSocket == null || !isBtConnected)
                {
                     myBluetooth = BluetoothAdapter.getDefaultAdapter();//get the mobile bluetooth device
                     BluetoothDevice dispositivo = myBluetooth.getRemoteDevice(address);//connects to the device's address and checks if it's available
                     btSocket = dispositivo.createInsecureRfcommSocketToServiceRecord(myUUID);//create a RFCOMM (SPP) connection
                     BluetoothAdapter.getDefaultAdapter().cancelDiscovery();
                     btSocket.connect();//start connection
                }
            }
            catch (IOException e)
            {
                ConnectSuccess = false;//if the try failed, you can check the exception here
            }
            return null;
        }

        @Override
        protected void onPostExecute(Void result) //after the doInBackground, it checks if everything went fine
        {
            super.onPostExecute(result);

            if (!ConnectSuccess)
            {
                msg("Connection Failed. Is it a SPP Bluetooth? Try again.");
                finish();
            }
            else
            {
                msg("Connected.");
                isBtConnected = true;

                BluetoothConnected(isBtConnected);

            }
            progress.dismiss();
        }
    }
}
